<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns="http://docbook.org/ns/docbook"
  xmlns:db="http://docbook.org/ns/docbook" xmlns:xlink="http://www.w3.org/1999/xlink"
  exclude-result-prefixes="xs" version="2.0">

  <xsl:param name="javaApiBaseUrl"
  >https://docs.oracle.com/en/java/javase/21/docs/api/java.base</xsl:param>

  <xsl:template name="hdm.get.leftmost.of.separator">
    <xsl:param name="path"/>
    <xsl:param name="separator"/>
    <xsl:param name="start">1</xsl:param>
    <xsl:if test="contains($path, $separator)">
      <xsl:if test="$start = '0'">
        <xsl:value-of select="$separator"/>
      </xsl:if>
      <xsl:value-of select="substring-before($path, $separator)"/>
      <xsl:call-template name="hdm.get.leftmost.of.separator">
        <xsl:with-param name="path" select="substring-after($path, $separator)"/>
        <xsl:with-param name="separator" select="$separator"/>
        <xsl:with-param name="start">0</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template match="db:qandaset" mode="selectOwnOrInheritedTitle">
    <link linkend="{@xml:id}">
      <!--  xrefstyle="template:%n, %t" -->
      <xsl:choose>
        <xsl:when test="db:title">
          <xsl:copy-of select="db:title/node()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="preceding-sibling::db:title[1]/node()"/>
        </xsl:otherwise>
      </xsl:choose>
    </link>
  </xsl:template>

  <xsl:template match="db:qandaset" mode="selectOwnOrInheritedTitleFlat">
    <xsl:choose>
      <xsl:when test="db:title">
        <xsl:apply-templates select="db:title" mode="flatten"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="preceding-sibling::db:title[1]" mode="flatten"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template match="*" mode="flatten">
    <xsl:apply-templates select="text() | *" mode="flatten"/>
  </xsl:template>

  <xsl:template match="text()" mode="flatten">
    <xsl:value-of select="normalize-space(.)"/>
    <xsl:text> </xsl:text>
  </xsl:template>


  <xsl:template match="@xlink:href">
    <xsl:attribute name="xlink:href">

      <xsl:choose>
        <xsl:when test="starts-with(., 'javaapi://')">

          <xsl:variable name="javaUrl">
            <xsl:value-of select="substring-after(., 'javaapi://')"/>
          </xsl:variable>

          <xsl:choose>
            <xsl:when test="contains($javaUrl, '#')">

              <xsl:variable name="fqJavaClassname" select="substring-before($javaUrl, '#')"/>

              <xsl:call-template name="javaApiClassname2htmlReference">
                <xsl:with-param name="fqJavaClassname" select="$fqJavaClassname"/>
              </xsl:call-template>
              <xsl:text>#</xsl:text>

              <xsl:variable name="javaClassname">
                <xsl:value-of select="tokenize($fqJavaClassname, '\.')[last()]"/>
                <!--
                <xsl:variable name="fqJavaNameComponents" select="tokenize($fqJavaClassname, '\.')"/>
                <xsl:value-of select="$fqJavaNameComponents[last()]"/>
                -->
              </xsl:variable>

              <xsl:call-template name="javaApiArgs2htmlReference">
                <xsl:with-param name="javaClassName" select="$javaClassname"/>
                <xsl:with-param name="javaMethodnameAndArguments" select="substring-after($javaUrl, '#')"/>
              </xsl:call-template>

            </xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="javaApiClassname2htmlReference">
                <xsl:with-param name="fqJavaClassname" select="$javaUrl"/>
              </xsl:call-template>
            </xsl:otherwise>

          </xsl:choose>
        </xsl:when>

        <xsl:otherwise>
          <xsl:attribute name="xlink:href" select="."/>
        </xsl:otherwise>

      </xsl:choose>
    </xsl:attribute>

  </xsl:template>

  <xsl:template name="javaApiArgs2htmlReference">
    <xsl:param name="javaClassName"/>
    <xsl:param name="javaMethodnameAndArguments"/>

    <xsl:variable name="methodname" select="substring-before($javaMethodnameAndArguments, '(')"/>

    <xsl:variable name="unescapedHtmlReference">
      <xsl:choose>
        <xsl:when test="$methodname eq $javaClassName">
          <xsl:text>&lt;init&gt;(</xsl:text>
          <xsl:value-of select="substring-after($javaMethodnameAndArguments, '(')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$javaMethodnameAndArguments"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <!--
    https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/lang/String.html#%3Cinit%3E(byte%5B%5D,int,int,java.nio.Charset)
    https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/lang/String.html#%3Cinit%3E(byte%5B%5D,int,int,java.nio.charset.Charset)
    -->

    <xsl:value-of select="replace(replace($unescapedHtmlReference,'\[','%5B'), '\]', '%5D')"/>
  </xsl:template>

  <xsl:template name="javaApiClassname2htmlReference">
    <xsl:param name="fqJavaClassname"/>
    <xsl:value-of select="$javaApiBaseUrl"/>
    <xsl:choose>
      <xsl:when test="contains($fqJavaClassname, '.')">
        <xsl:text>/</xsl:text>
        <xsl:value-of select="replace($fqJavaClassname, '\.', '/')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>/java/lang/</xsl:text>
        <xsl:value-of select="$fqJavaClassname"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>.html</xsl:text>
  </xsl:template>

</xsl:stylesheet>
